Stan Math Library: stan/math/prim/scal/prob/beta_ccdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

beta_ccdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_BETA_CCDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_BETA_CCDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_less_or_equal.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
12 #include <stan/math/prim/scal/fun/log1m.hpp>
13 #include <stan/math/prim/scal/fun/multiply_log.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <stan/math/prim/scal/fun/digamma.hpp>
16 #include <stan/math/prim/scal/fun/lgamma.hpp>
17 #include <stan/math/prim/scal/fun/lbeta.hpp>
18 #include <stan/math/prim/scal/meta/contains_nonconstant_struct.hpp>
19 #include <stan/math/prim/scal/meta/VectorView.hpp>
20 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
21 #include <stan/math/prim/scal/fun/constants.hpp>
22 #include <stan/math/prim/scal/meta/include_summand.hpp>
23 #include <stan/math/prim/scal/fun/grad_reg_inc_beta.hpp>
24 #include <stan/math/prim/scal/fun/inc_beta.hpp>
25 #include <boost/math/special_functions/gamma.hpp>
26 #include <boost/random/gamma_distribution.hpp>
27 #include <boost/random/variate_generator.hpp>
28 #include <cmath>
29
30 namespace stan {
31
32 namespace math {
33
34 template <typename T_y, typename T_scale_succ, typename T_scale_fail>
35 typename return_type<T_y, T_scale_succ, T_scale_fail>::type
36 beta_ccdf_log(const T_y& y, const T_scale_succ& alpha,
37 const T_scale_fail& beta) {
38 typedef typename stan::partials_return_type<T_y, T_scale_succ,
39 T_scale_fail>::type
40 T_partials_return;
41
42 // Size checks
43 if ( !( stan::length(y) && stan::length(alpha)
44 && stan::length(beta) ) )
45 return 0.0;
46
47 // Error checks
48 static const char* function("stan::math::beta_cdf");
49
50 using stan::math::check_positive_finite;
51 using stan::math::check_not_nan;
52 using stan::math::check_nonnegative;
53 using stan::math::check_less_or_equal;
54 using boost::math::tools::promote_args;
55 using stan::math::check_consistent_sizes;
56 using stan::math::value_of;
57
58 T_partials_return ccdf_log(0.0);
59
60 check_positive_finite(function, "First shape parameter", alpha);
61 check_positive_finite(function, "Second shape parameter", beta);
62 check_not_nan(function, "Random variable", y);
63 check_nonnegative(function, "Random variable", y);
64 check_less_or_equal(function, "Random variable", y, 1);
65 check_consistent_sizes(function,
66 "Random variable", y,
67 "First shape parameter", alpha,
68 "Second shape parameter", beta);
69
70 // Wrap arguments in vectors
71 VectorView<const T_y> y_vec(y);
72 VectorView<const T_scale_succ> alpha_vec(alpha);
73 VectorView<const T_scale_fail> beta_vec(beta);
74 size_t N = max_size(y, alpha, beta);
75
76 OperandsAndPartials<T_y, T_scale_succ, T_scale_fail>
77 operands_and_partials(y, alpha, beta);
78
79 // Compute CDF and its gradients
80 using stan::math::inc_beta;
81 using stan::math::digamma;
82 using stan::math::lbeta;
83 using std::pow;
84 using std::exp;
85 using std::log;
86 using std::exp;
87
88 // Cache a few expensive function calls if alpha or beta is a parameter
89 VectorBuilder<contains_nonconstant_struct<T_scale_succ,
90 T_scale_fail>::value,
91 T_partials_return, T_scale_succ, T_scale_fail>
92 digamma_alpha_vec(max_size(alpha, beta));
93 VectorBuilder<contains_nonconstant_struct<T_scale_succ,
94 T_scale_fail>::value,
95 T_partials_return, T_scale_succ, T_scale_fail>
96 digamma_beta_vec(max_size(alpha, beta));
97 VectorBuilder<contains_nonconstant_struct<T_scale_succ,
98 T_scale_fail>::value,
99 T_partials_return, T_scale_succ, T_scale_fail>
100 digamma_sum_vec(max_size(alpha, beta));
101
102 if (contains_nonconstant_struct<T_scale_succ, T_scale_fail>::value) {
103 for (size_t i = 0; i < N; i++) {
104 const T_partials_return alpha_dbl = value_of(alpha_vec[i]);
105 const T_partials_return beta_dbl = value_of(beta_vec[i]);
106
107 digamma_alpha_vec[i] = digamma(alpha_dbl);
108 digamma_beta_vec[i] = digamma(beta_dbl);
109 digamma_sum_vec[i] = digamma(alpha_dbl + beta_dbl);
110 }
111 }
112
113 // Compute vectorized CDFLog and gradient
114 for (size_t n = 0; n < N; n++) {
115 // Pull out values
116 const T_partials_return y_dbl = value_of(y_vec[n]);
117 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
118 const T_partials_return beta_dbl = value_of(beta_vec[n]);
119 const T_partials_return betafunc_dbl = exp(lbeta(alpha_dbl, beta_dbl));
120
121 // Compute
122 const T_partials_return Pn = 1.0 - inc_beta(alpha_dbl, beta_dbl, y_dbl);
123
124 ccdf_log += log(Pn);
125
126 if (!is_constant_struct<T_y>::value)
127 operands_and_partials.d_x1[n] -= pow(1-y_dbl, beta_dbl-1)
128 * pow(y_dbl, alpha_dbl-1) / betafunc_dbl / Pn;
129
130 T_partials_return g1 = 0;
131 T_partials_return g2 = 0;
132
133 if (contains_nonconstant_struct<T_scale_succ, T_scale_fail>::value) {
134 stan::math::grad_reg_inc_beta(g1, g2, alpha_dbl, beta_dbl, y_dbl,
135 digamma_alpha_vec[n],
136 digamma_beta_vec[n],
137 digamma_sum_vec[n],
138 betafunc_dbl);
139 }
140 if (!is_constant_struct<T_scale_succ>::value)
141 operands_and_partials.d_x2[n] -= g1 / Pn;
142 if (!is_constant_struct<T_scale_fail>::value)
143 operands_and_partials.d_x3[n] -= g2 / Pn;
144 }
145
146 return operands_and_partials.value(ccdf_log);
147 }
148 }
149 }
150 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
lgamma.hpp
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::lbeta
fvar< T > lbeta(const fvar< T > &x1, const fvar< T > &x2)
Definition: lbeta.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
contains_nonconstant_struct.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::beta_ccdf_log
return_type< T_y, T_scale_succ, T_scale_fail >::type beta_ccdf_log(const T_y &y, const T_scale_succ &alpha, const T_scale_fail &beta)
Definition: beta_ccdf_log.hpp:36
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::inc_beta
fvar< T > inc_beta(const fvar< T > &a, const fvar< T > &b, const fvar< T > &x)
Definition: inc_beta.hpp:20
stan::partials_return_type
Definition: partials_return_type.hpp:16
check_less_or_equal.hpp
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
log1m.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_less_or_equal
bool check_less_or_equal(const char *function, const char *name, const T_y &y, const T_high &high)
Return true if y is less or equal to high.
Definition: check_less_or_equal.hpp:81
stan::contains_nonconstant_struct
Definition: contains_nonconstant_struct.hpp:14
grad_reg_inc_beta.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
lbeta.hpp
inc_beta.hpp
stan::math::pow
fvar< T > pow(const fvar< T > &x1, const fvar< T > &x2)
Definition: pow.hpp:18
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::math::grad_reg_inc_beta
void grad_reg_inc_beta(T &g1, T &g2, T a, T b, T z, T digammaA, T digammaB, T digammaSum, T betaAB)
Definition: grad_reg_inc_beta.hpp:14
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.